#!/bin/bash

set -v

#N - number of samples
N=1100
#C - number of children (observed variables)
C=818
#L - number of latent variables (only used to generate samples, not necessary for learning)  
L=2 
#T - threshold for pairs. Only consider top T pairs
T=5000
#SEED - random seed, not used but shows up in path names. Keep this as 0
SEED=0
#eig - threshold for maximum 3rd eigenvalue to be considered a *singly* - coupled quartet
#also called Tau_q in the paper
eig=0.01 #0.01
#coup - threshold for minimum conditional mutual information to be considered a *coupled* quartet
#also called Tau_e in the paper
coup=0.001 #0.001


#compile sampling code
cd sampling
make
cd -

#generate some data. If you have your own data, skip this step!
python generate_data.py $N $C $L 0.7 true_model.txt data.dat

#make a directory for the new network
python initialize_network.py data.dat test_network

#extract pairwise sufficient statistics
python write_schedule.py networks/test_network/true_params/schedule2.txt $C 2
sampling/read_ss networks/test_network networks/test_network/samples/data.dat schedule2.txt $N $C

#filter quartets to include only pairs with high mutual information
python filter_quartets.py networks/test_network $T $N $SEED schedule2.txt schedule4.txt
sampling/read_ss networks/test_network networks/test_network/samples/data.dat schedule4.txt $N $C

#learn the structure of the network using these samples
python main.py networks/test_network schedule4.txt $SEED $N $eig $coup $T

#visualize the learned network
python visualize_network.py networks/test_network $eig $coup $N out.dot

dot -Tpdf out.dot > out.pdf
open out.pdf
