import numpy as np
import itertools
import sys

def log_sum_exp(l, signs):
    M = l.max()
    l -= M
    l = np.exp(l)
    return M + np.log(np.dot(l, signs))

def prior_prob(priors, p_state):
    P = 1
    for p in xrange(len(priors)):
        if p_state[p] == 0:
            P *= 1-priors[p]
        else:
            P *= priors[p]
    return P

def _prob(p_state, state, f, n):
    P = n
    for p in xrange(len(p_state)):
        if p_state[p] == 1:
            P *= f[p]

    if state == 1:
        P = 1-P
    return P

def prob(p_state, failures, noise):
    P = np.zeros((2,2,2,2))
    numChildren = len(failures[0])
    assert(numChildren == 4)
    failures = np.array(failures)
    for c_state in itertools.product([0,1], repeat = numChildren):
        #print "c_state", c_state
        P[c_state] += _prob(p_state, c_state[0], failures[:, 0], noise[0])*_prob(p_state, c_state[1], failures[:, 1], noise[1]) * _prob(p_state, c_state[2], failures[:, 2], noise[2])*_prob(p_state, c_state[3], failures[:, 3], noise[3])

    return P


def jointProbability(priors, failures, noise):

    P = np.zeros((2,2,2,2))
    numParents = len(priors)
    for p_state in itertools.product([0,1], repeat = numParents):
        P += prior_prob(priors, p_state)*prob(p_state, failures, noise)
    return P

def influence(p, F):
    #p and f are in log space#
    assert p < 0
    assert (F <= 0).all()
    combined_failure = 0
    for f in F:
        combined_failure += f
    return log_sum_exp(np.array([0, p, p+combined_failure]), np.array([1, -1, 1]))

def _params_to_moment_helper(priors,failures,noise, indices):
    #p,f,n are in log space#
    assert (priors < 0).all()
    assert (failures <= 0).all()
    assert (noise <= 0).all()

    numChildren = len(indices)
    numParents = priors.size
    M = 0
    for p in xrange(numParents):
        M += influence(priors[p],failures[p, :][:, indices])
    for c in xrange(numChildren):
        M += noise[indices[c]]
    return M
    
def params_to_moments(p,f,n):
    assert (p < 0).all()
    assert (f <= 0).all()
    assert (n <= 0).all()
    #M is in log space#
    M = np.zeros(7)
    M[0] = _params_to_moment_helper(p, f, n, [0])
    M[1] = _params_to_moment_helper(p, f, n, [1])
    M[2] = _params_to_moment_helper(p, f, n, [2])

    M[3] = _params_to_moment_helper(p, f, n, [0,1])
    M[4] = _params_to_moment_helper(p, f, n, [1,2])
    M[5] = _params_to_moment_helper(p, f, n, [0,2])

    M[6] = _params_to_moment_helper(p, f, n, [0,1,2])
    return M

def dist_to_moments(D):
    #dist is never in log space
    assert (D >= 0).all()
    print "::dist_to_moments::"
    assert D.shape == (2,2,2)
    M= np.zeros(7)
    M[0] = D[0, :, :].sum()
    M[1] = D[:, 0, :].sum()
    M[2] = D[:, :, 0].sum()

    M[3] = D[0, 0, :].sum()
    M[4] = D[:, 0, 0].sum()
    M[5] = D[0, :, 0].sum()

    M[6] = D[0,0,0]
    return np.log(M)


def moments_to_dist(M, logspace):
    print "::moments_to_dist::"
    assert M.shape == (7,)

    if logspace:
        M = np.exp(M)

    assert (M >= 0).all()
    D = np.zeros((2,2,2))

    D[0,0,0] = M[6]
    
    D[0,0,1] = M[3] - M[6]
    D[0,1,0] = M[5] - M[6]
    D[1,0,0] = M[4] - M[6]

    D[1,0,1] = M[1] - D[1,0,0] - D[0,0,1] - D[0,0,0]
    D[1,1,0] = M[2] - D[1,0,0] - D[0,1,0] - D[0,0,0]
    D[0,1,1] = M[0] - D[0,1,0] - D[0,0,1] - D[0,0,0]

    D[1,1,1] = 1- D.sum()
    assert (D >= 0).all()
    return D

def samples_to_dist(data):
    N, numChildren = data.shape
    assert(numChildren == 3)
    D = np.zeros((2,2,2))
    for n in xrange(N):
        D[tuple(data[n, :])] += 1.0
    D /= N
    return D
    
    
