#include <iostream>
#include <fstream>
#include "statistics.h"
#include <stdlib.h>
#include "mtrand.h"

using namespace std;

Stats::Stats(char* dirname, int sample_seed, char* scheduleName, int numSamples, int C, char* raw_samples){
    verbose=false;
    counter = NULL;
    this->C = C;
    seed = sample_seed;
    network_name = dirname;
    schedule_name = scheduleName;
    raw_samples_name = raw_samples;
    num_samples = numSamples;
}

void Stats::getSchedule()
{
    char* dirname = network_name;
    char fname[100];
    ifstream f;
    
    sprintf(fname, "%s/true_params/%s", dirname, schedule_name);
    f.open(fname);
    f >> N;

    schedule = new Quartet[N];

    for(int i = 0; i < N; i++)
    {
        f >> schedule[i].a;
        f >> schedule[i].b;
        f >> schedule[i].c;
        f >> schedule[i].d;
    }
    f.close();


    printf("initialized stats reader\n");   
}

void Stats::initialize_counter()
{
    if(counter) delete counter;
    counter = new int[16*N];
    for(int i = 0; i < 16*N; i++) counter[i] = 0;
    return;
}

void Stats::read()
{
    bool reading = true;
    MTRand rnd;
	rnd.seed((unsigned long int)seed);
    printf("reading\n");
    char* dirname = network_name;
    int obs[C];
    float r;

    ofstream out;
    char fname[256];
    sprintf(fname, "%s/progress_log_n%d_s%d_%s", network_name, num_samples, seed, schedule_name);
    out.open(fname);

    ifstream in;
    if(reading)
    {
        printf("reading samples from %s\n", raw_samples_name);
        in.open(raw_samples_name);
    }

    for(int sample_counter = 0; sample_counter < num_samples; sample_counter++)
    {
        if((sample_counter  % 500) ==0)
        {
            printf("sample %d\n", sample_counter);
        }

        for(int c=0; c<C; c++)
        {
            in >> obs[c];
        }
    

        //time to update counter

        int a,b,c,d;
        for(int s = 0; s < N; s++)
        {
            a = (schedule[s].a >= 0? obs[schedule[s].a]:0);
            b = (schedule[s].b >= 0? obs[schedule[s].b]:0);
            c = (schedule[s].c >= 0? obs[schedule[s].c]:0);
            d = (schedule[s].d >= 0? obs[schedule[s].d]:0);
            counter[16*s + 8*a + 4*b + 2*c+d] += 1;
        }
    }
    out.close();
}


void Stats::printStats()
{
    ofstream out;
    char fname[256];
    sprintf(fname, "%s/samples/sufficientStatistics_n%d_s%d_%s", network_name, num_samples, seed, schedule_name);
    cout << "printing sufficient statistics to " << fname << endl;

    out.open(fname);
    out << C << endl;
    cout << "here" << endl;
    for(int i = 0; i < 16*N; i++){
        if(i>0 && i%16==0) out << endl;
        out << counter[i] << " ";
    }
    out.close();
    
    sprintf(fname, "%s/samples/id_n%d_s%d_%s", network_name, num_samples, seed, schedule_name);
    out.open(fname);
    for(int i = 0; i < N; i++){
        out << schedule[i].a << " "<< schedule[i].b << " " << schedule[i].c << " " << schedule[i].d << endl;;
    }
    out.close();
}

